%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
%% @doc Standard functions on booleans. In Erlang, boolean values are
%% represented by the atoms <code>true</code> and <code>false</code>.

-module(erl.lang.bool).

-export(['and'/2,
	 'not'/1,
	 'or'/2,
	 'xor'/2]).

%% @doc Returns the logical conjunction (<em>and</em>)of two booleans.

'and'(false, _) -> false;
'and'(_, false) -> false;
'and'(true, true) -> true;
'and'(_, _) -> erl.lang:error(badarg).

%% @doc Returns the logical negation (<em>not</em>) of a boolean.

'not'(true) -> false;
'not'(false) -> true;
'not'(_) -> erl.lang:error(badarg).

%% @doc Returns the logical disjunction (<em>or</em>) of two booleans.

'or'(true, _) -> true;
'or'(_, true) -> true;
'or'(false, false) -> false;
'or'(_, _) -> erl.lang:error(badarg).

%% @doc Returns the logical exclusive-or of two booleans.

'xor'(true, false) -> true;
'xor'(true, true) -> false;
'xor'(false, true) -> true;
'xor'(false, false) -> false;
'xor'(_, _) -> erl.lang:error(badarg).
