%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(erl.lang.package).

-export([to_string/1, concat/1, concat/2, is_valid/1, is_segmented/1,
	 split/1, last/1,  first/1, strip_last/1, find_modules/1,
	 find_modules/2]).

%% @doc Return the package name as a string.
to_string(Name) -> packages:to_string(Name).

%% @doc Concatenate two names, inserting period characters as necessary.
%% @equiv concat([A, B])
concat(A, B) -> packages:concat(A, B).

%% @doc Concatenate a list of names, inserting period characters as
%% necessary.  If a segment is empty, the result might not be a valid
%% name; {@link is_valid/1} can be used to check the result.
concat(L) -> packages:concat(L).

%% @doc Returns <code>true</code> if the name is a valid package name,
%% otherwise <code>false</code>.
is_valid(Name) -> packages:is_valid(Name).

%% @doc Returns <code>true</code> if the name has more than one segment,
%% otherwise <code>false</code>.
is_segmented(Name) -> packages:is_segmented(Name).

%% @doc Returns the list of segments of a name.
split(Name) -> packages:split(Name).

%% @doc Returns the last segment of a name.
last(Name) -> packages:last(Name).

%% @doc Returns the list of all segments but the last of a name.
first(Name) -> packages:first(Name).

%% @doc Returns the name with its last segment removed.
strip_last(Name) -> packages:strip_last(Name).

%% @doc Finds all modules available for a given package, using the
%% current code server search path. There is no guarantee that the
%% modules can be loaded; only that the object files exist.
%% @see find_modules/2

find_modules(P) -> packages:find_modules(P).

%% @doc Finds all modules available for a given package, using the
%% specified search path.
%% @see find_modules/1

find_modules(P, Paths) -> packages:find_modules(P, Paths).
